/**********************************************************************
Copyright (c) 2011 Andy Jefferson and others. All rights reserved.
Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.

Contributors:
   ...
**********************************************************************/
package org.datanucleus.store.mongodb.query.expression;

import com.mongodb.BasicDBObject;
import org.datanucleus.exceptions.NucleusException;

/**
 * Representation of a boolean expression in MongoDB queries.
 * Has the DBObject that it represents.
 */
public class MongoBooleanExpression extends MongoExpression
{
    BasicDBObject dbObject = null;

    /**
     * Constructor when the expression represents a comparison, between the field and a value.
     * @param fieldName Name of the document field
     * @param value The value
     * @param op The operator (eq, noteq, lt, gt, etc)
     */
    public MongoBooleanExpression(String fieldName, Object value, MongoOperator op)
    {
        if (op == MongoOperator.OP_EQ)
        {
            dbObject = new BasicDBObject(fieldName, value);
        }
        else if (op == MongoOperator.OP_NOTEQ || op == MongoOperator.OP_LT || op == MongoOperator.OP_LTEQ ||
            op == MongoOperator.OP_GT || op == MongoOperator.OP_GTEQ)
        {
            BasicDBObject valObject = new BasicDBObject(op.getValue(), value);
            dbObject = new BasicDBObject(fieldName, valObject);
        }
        else
        {
            throw new NucleusException("Cannot create MongoBooleanExpression with operator of " + op + " with this constructor");
        }
    }

    public MongoBooleanExpression(MongoBooleanExpression expr1, MongoBooleanExpression expr2, MongoOperator op)
    {
        if (op == MongoOperator.OP_AND)
        {
            BasicDBObject[] andOptions = new BasicDBObject[2];
            andOptions[0] = expr1.getDBObject();
            andOptions[1] = expr2.getDBObject();
            dbObject = new BasicDBObject(op.getValue(), andOptions);
        }
        else if (op == MongoOperator.OP_OR)
        {
            BasicDBObject[] orOptions = new BasicDBObject[2];
            orOptions[0] = expr1.getDBObject();
            orOptions[1] = expr2.getDBObject();
            dbObject = new BasicDBObject(op.getValue(), orOptions);
        }
        else
        {
            throw new NucleusException("Cannot create MongoBooleanExpression with operator of " + op + " with this constructor");
        }
    }

    public BasicDBObject getDBObject()
    {
        return dbObject;
    }

    public String toString()
    {
        return dbObject.toString();
    }
}